/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <stdlib.h>
#include <zephyr.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/net/openthread.h>
#include <openthread/coap.h>
#include <openthread/ip6.h>
#include <openthread/message.h>
#include <openthread/thread.h>



//data functions
static void temperature_request_handler(void *p_context, otMessage *p_pessage, const otMessageInfo *p_message_info);
static otError temperature_response_send(otMessage *p_request_message, const otMessageInfo *p_request_message_info);

float mTemperature = 23.32;
char *T;

//coap resource implementation
static otCoapResource m_send_temperature_resource ={
	.mUriPath = "temperature",
	.mHandler = temperature_request_handler, //callback for handling a received request
	.mContext = NULL,
	.mNext = NULL
};

static void temperature_request_handler(void *p_context, otMessage *p_message, const otMessageInfo *p_message_info)
{
	otError error;
	printk("Received temperature request\n");

	//otCoapCode messageCode = otCoapMessageGetCode(p_message); 
	//otCoapType messageType = otCoapMessageGetType(p_message);

	if ((otCoapMessageGetType(p_message) == OT_COAP_TYPE_CONFIRMABLE) && (otCoapMessageGetCode(p_message) == OT_COAP_CODE_GET))
	{
		error = temperature_response_send(p_message, p_message_info);
		if (error == OT_ERROR_NONE)
		{
			printk("Sent response\n");
		}
	}
}

static otError temperature_response_send(otMessage *p_request_message, const otMessageInfo *p_request_message_info)
{
	otError error = OT_ERROR_NO_BUFS;
	otMessage *p_response;											//create a message variable									
	otInstance *p_instance = openthread_get_default_instance();   	//create a new instance
	printk("Conversion\n");
	gcvt (mTemperature, 4, T);
	printk("Temperature: %s\n", T);
	p_response = otCoapNewMessage(p_instance, NULL);				//create a new message
	if (p_response == NULL) {
		printk("Error while creating a new message\n");
		goto end;
	}
	
	otCoapMessageInit(p_response, OT_COAP_TYPE_CONFIRMABLE, OT_COAP_CODE_CONTENT);
	if (p_response == NULL) {
		printk("Error while initializing the response message: %d\n", error);
		goto end;
	}

	error = otCoapMessageInitResponse(p_response, p_request_message, OT_COAP_TYPE_ACKNOWLEDGMENT, OT_COAP_CODE_CONTENT);
	if (error != OT_ERROR_NONE) { 
		printk("Failed to init response: %d\n", error); 
		}

	error = otCoapMessageSetToken(p_response, otCoapMessageGetToken(p_request_message), otCoapMessageGetTokenLength(p_request_message));
	if (error != OT_ERROR_NONE) {
		printk("Error while setting the Token: %d\n", error);
		goto end;
	}
	/*
	error = otCoapMessageAppendContentFormatOption(p_response, OT_COAP_OPTION_CONTENT_FORMAT_JSON);
	if (error != OT_ERROR_NONE) {
		printk("Error while setting format of the message"); 
		break; 
	}
	*/
	error = otCoapMessageSetPayloadMarker(p_response);
	if (error != OT_ERROR_NONE) {
		printk("Error while setting the payload start: %d\n", error);
		goto end;
	}
	error = otMessageAppend(p_response, T, strlen(T));
	if (error != OT_ERROR_NONE) { 
		printk("Error while appendig message: %d\n", error);
		goto end;
	}

	error = otCoapSendResponse(p_instance, p_response, p_request_message_info);	
	if (error != OT_ERROR_NONE) {
		printk("Failed to send store data response: %d\n", error);
		goto end;
	}

	end:
	if (error != OT_ERROR_NONE && p_response != NULL) {
		otMessageFree(p_response);
	}
	return error;
}

//initializing coap
void coap_init(void)
{
	otError error;
	otInstance *p_instance = openthread_get_default_instance();
	m_send_temperature_resource.mContext = p_instance;

	do
	{
		error = otCoapStart(p_instance, OT_DEFAULT_COAP_PORT);
		if (error != OT_ERROR_NONE){
			break;
		}

		otCoapAddResource(p_instance, &m_send_temperature_resource);
		
	} while (false);

	if (error != OT_ERROR_NONE){
		printk("coap_init_error: %d\n", error);
	}
}

void main(void)
{
	printk("Running server\n");
	coap_init();
}
